import traceback
from contextlib import contextmanager, ContextDecorator
import uuid
from enum import Enum
from filelock import FileLock

from b_logger.entities.attachments import Attachment
from b_logger.entities.prints import Print
from b_logger.utils.paths import b_logs_tmp_path, b_logs_tmp_steps_path
from b_logger.utils.basedatamodel import BaseDataModel
from b_logger.utils.formatters import format_exc, format_tb


class StepStatus(str, Enum):
    PASSED = 'passed'
    FAILED = 'failed'
    SKIPPED = 'skipped'
    WARNING = 'warning'
    NONE = 'none'


# class StepManager:
#     def __init__(self):
#         self.current_step_id = None
#         self.failed = False
#
#     def set_current_step(self, step_id):
#         self.current_step_id = step_id


class StepError(BaseDataModel):
    def __init__(self, exc, tb=None):
        self.exc = format_exc(exc)
        self.tb = tb or format_tb(traceback.format_exc(4))


class Step(BaseDataModel):
    def __init__(self,
                 title: str = None,
                 status: StepStatus = None,
                 expected: str = None
                 ):
        self.id = f'step_{uuid.uuid4()}'
        self.title = title
        self.status = status
        self.expected = expected
        self.parent_id = None
        self.error: StepError | None = None
        self.info = {}
        self.attachments = []
        self.known_bugs = []
        self.steps = []

    def set_parent_id(self, parent_id):
        self.parent_id = parent_id

    def add_attachment(self, attachment: Attachment):
        self.attachments.append(attachment)

    def set_status(self, status: StepStatus):
        self.status = status

    def set_error(self, error: StepError):
        self.error = error

    def add_sub_step(self, step):
        self.steps.append(step)

    def add_info(self, info: dict):
        for key, value in info.items():
            if key in self.info:
                existing = self.info[key]
                if not isinstance(existing, list):
                    self.info[key] = [existing]
                self.info[key].append(value)
            else:
                self.info[key] = value

    def add_known_bug(self, bug):
        self.known_bugs.append(bug)

    def get_sub_steps(self, as_dict: bool = False):
        if as_dict:
            return {step.id: step for step in self.steps}
        else:
            return self.steps


class StepContainer(BaseDataModel, dict):
    def __init__(self):
        super().__init__()
        self.container_id = f'steps_{uuid.uuid4()}'
        self.current_step_id = None
        self.failed = False
        self.current_stage = 'setup'
        self['setup'] = []
        self['call'] = []
        self['teardown'] = []

    def set_current_step(self, step_id):
        self.current_step_id = step_id

    def add_step(self, step: Step | Print):
        cur_stg = self.get(self.current_stage)
        cur_stg.append(step)

    def get_all_steps(self) -> list[Step]:
        return self.get('setup', []) + self.get('call', []) + self.get('teardown', [])

    def get_current_step(self) -> Step | None:
        return self.get_step_by_id(self.current_step_id)

    def get_step_by_id(self, step_id: str) -> Step | None:
        for step in self.get_all_steps():
            found = self._recursive_search(step, step_id)
            if found:
                return found
        return None

    @staticmethod
    def _recursive_search(step: Step, step_id: str) -> Step | None:
        if step.id == step_id:
            return step
        for sub_step in step.steps:
            if not sub_step.id.startswith('print_'):
                found = StepContainer._recursive_search(sub_step, step_id)
                if found:
                    return found
        return None

    def save_json(self, file_name=None):
        root = f'{b_logs_tmp_steps_path()}'
        if file_name:
            path = f'{root}/{file_name}'
        else:
            path = f'{root}/{self.container_id}'
        with FileLock(f'{path}.lock'):
            self.to_json_file(path)



import json
from jinja2 import Environment, FileSystemLoader

from b_logger.entities.reports import RunReport
from b_logger.utils.paths import pathfinder, b_logs_path


class HTMLGenerator:
    def __init__(self):
        env = Environment(loader=FileSystemLoader(f'{pathfinder.library_root()}/b_logger/templates'))
        self.template = env.get_template(f'base_template.html')
        self.summary_template = env.get_template(f'summary_template.html')
        self.report_path = f'{b_logs_path()}/blog_report.json'

    def generate_html(self):
        combined_report = RunReport.from_json(self.report_path)

        steps = combined_report.get_steps()

        html = self.template.render(
            report=combined_report,
            steps=steps
        )

        with open(f'{b_logs_path()}/blog_report.html', 'w', encoding='utf-8') as f:
            f.write(html)

        html_summary = self.summary_template.render(
            report=combined_report
        )

        with open(f'{b_logs_path()}/blog_summary.html', 'w', encoding='utf-8') as f:
            f.write(html_summary)



from enum import Enum


class TestStatus(str, Enum):
    PASSED = 'PASSED'
    FAILED = 'FAILED'
    SKIPPED = 'SKIPPED'
    BROKEN = 'BROKEN'
    NONE = 'NONE'


def py_outcome_to_tstatus(outcome):
    return {
        'passed': TestStatus.PASSED,
        'failed': TestStatus.FAILED,
        'skipped': TestStatus.SKIPPED,
        'broken': TestStatus.BROKEN,
    }.get(outcome, TestStatus.NONE)



# def decor1(func):
#     def wrapper(*args, **kwargs):
#         print("Вызван decor1")
#         return func(*args, **kwargs)
#     return wrapper
#
#
# def decor2(func):
#     def wrapper(*args, **kwargs):
#         print("Вызван decor2")
#         return func(*args, **kwargs)
#     return wrapper
#
#
# def apply_decorators(*decorators):
#     def meta_decorator(func):
#         for decorator in reversed(decorators):  # Применяем в порядке вызова
#             func = decorator(func)
#         return func
#     return meta_decorator
#
#
# @apply_decorators(decor1, decor2)
# def my_function():
#     print("Оригинальная функция")
#
# my_function()

# @contextmanager
# def log_context(name):
#     print(f"Начало {name}")
#     yield
#     print(f"Конец {name}")
#
# def context_decorator(context_manager):
#     def wrapper(func):
#         @wraps(func)
#         def wrapped(*args, **kwargs):
#             with context_manager(func.__name__):
#                 return func(*args, **kwargs)
#         return wrapped
#     return wrapper
#
# @context_decorator(log_context)
# def my_function():
#     print("Выполнение функции")
#
# my_function()

# class log_context(ContextDecorator):
#     def __init__(self, name):
#         self.name = name
#
#     def __enter__(self):
#         print(f"Начало {self.name}")
#         return self
#
#     def __exit__(self, exc_type, exc_value, traceback):
#         print(f"Конец {self.name}")
#
# @log_context("my_function")
# def my_function():
#     print("Выполнение функции")
#
# my_function()
#
#
# import functools
# from contextlib import ContextDecorator as PyContextDecorator
# from contextlib import _GeneratorContextManager as GeneratorContextManager
#
#
# class ContextManager(GeneratorContextManager, PyContextDecorator):
#     """Pass in a generator to the initializer and the resultant object
#     is both a decorator closure and context manager
#     """
#
#     def __init__(self, func, args=(), kwargs=None):
#         if kwargs is None:
#             kwargs = {}
#
#         super().__init__(func, args, kwargs)
#
#
# def contextdecorator(func):
#     """Similar to contextlib.contextmanager except the decorated generator
#     can be used as a decorator with optional arguments.
#     """
#
#     @functools.wraps(func)
#     def helper(*args, **kwargs):
#         is_decorating = len(args) == 1 and callable(args[0])
#
#         if is_decorating:
#             new_func = args[0]
#
#             @functools.wraps(new_func)
#             def new_helper(*args, **kwargs):
#                 instance = ContextManager(func)
#                 return instance(new_func)(*args, **kwargs)
#
#             return new_helper
#         return ContextManager(func, args, kwargs)
#
#     return helper

# import functools
# from contextlib import ContextDecorator, contextmanager
# from typing import Callable, Generator, Optional, Any
#
# class UniversalContext(ContextDecorator):
#     """Объединяет контекстный менеджер и декоратор"""
#
#     def __init__(self, func: Callable[..., Generator], *args: Any, **kwargs: Any):
#         self.func = func
#         self.args = args
#         self.kwargs = kwargs
#
#     def __enter__(self):
#         """Вход в контекст"""
#         self.gen = self.func(*self.args, **self.kwargs)  # Создаем генератор
#         return next(self.gen)  # Переход к yield
#
#     def __exit__(self, exc_type, exc_value, traceback):
#         """Выход из контекста"""
#         try:
#             next(self.gen)  # Продолжаем выполнение после yield
#         except StopIteration:
#             pass  # Ожидаемое завершение
#
# def contextdecorator(func: Callable[..., Generator]) -> Callable[..., Any]:
#     """Декоратор, который превращает функцию в универсальный контекст"""
#
#     @functools.wraps(func)
#     def wrapper(*args, **kwargs):
#         # Проверяем, вызывается ли декоратор как `@contextdecorator` или `@contextdecorator()`
#         if len(args) == 1 and callable(args[0]):
#             return UniversalContext(func)(args[0])  # Без параметров: оборачиваем функцию
#         return UniversalContext(func, *args, **kwargs)  # С параметрами: создаем экземпляр
#
#     return wrapper


def format_exc(exc):
    exc = str(exc).partition('Stacktrace')[0]
    return exc


def format_tb(tb):
    tb = ('\n' +
          str(tb)
          .partition('Stacktrace')[0]
          # .partition('Other element would receive the click:')[0]
          + '\n')
    return tb



import sys
import os
import shutil
from pathlib import Path
from functools import lru_cache, wraps
from typing import Optional, Union, Any, Set


class PathFinder:
    """
    Utility for defining library and project roots and for file search

    Usage:
        pathfinder = PathResolver()
        project_root = pathfinder.project_root()
        library_root = pathfinder.library_root()
        path = pathfinder.find('settings.py', from_root='project')
    """

    def __init__(
            self,
            project_markers: Set[str] = None,
            library_markers: Set[str] = None,
    ):
        # Маркеры, определяющие корень проекта
        self.project_markers = project_markers or {'blog.config.yaml', 'requirements.txt', 'conftest.py'}
        # Маркеры, определяющие корень библиотеки
        self.library_markers = library_markers or {'plugin.py', 'b_logger'}

    @staticmethod
    def _get_possible_roots(cur_file_dir: str) -> Set[str]:
        """
        Returns possible roots based on current path ans sys.path"""
        possible_roots = set()
        cur_file_components = os.path.normpath(cur_file_dir).split(os.sep)

        for path in sys.path:
            possible_root_dir = True
            for cur_file_comp, comp in zip(cur_file_components, os.path.normpath(path).split(os.sep)):
                if cur_file_comp != comp:
                    possible_root_dir = False
            if possible_root_dir:
                possible_roots.add(path)

        return possible_roots

    @lru_cache(maxsize=1)
    def project_root(self) -> Path:
        """
        Find project root
        """
        current = Path.cwd().resolve()

        # Search directory up
        for parent in [current, *current.parents]:
            if any((parent / marker).exists() for marker in self.project_markers):
                return parent

        # Search through sys.path
        for path in sys.path:
            p = Path(path).resolve()
            if any((p / marker).exists() for marker in self.project_markers):
                return p

        raise RuntimeError(f"Unable to find project root. Markers to search through: {self.project_markers}")

    @lru_cache(maxsize=1)
    def library_root(self) -> Path:
        """
        Find library root
        """
        path = Path(__file__).resolve()
        possible_roots = self._get_possible_roots(str(path))

        for root in possible_roots:
            if any((Path(root) / marker).exists() for marker in self.library_markers):
                return Path(root)

        raise RuntimeError(f'Unable to find library root. '
                           f'Probably you forgot to add "blog.config.yaml" to your project')

    def find(
            self,
            name: str,
            from_root: str = 'project',
            include_dirs: bool = True,
            exclude_dirs: tuple[str, ...] = ('.git', '.venv', '__pycache__'),
    ) -> Union[str, Path]:
        """
        Search for file by its name
        """
        if from_root == 'project':
            root_path = self.project_root()
        elif from_root == 'library':
            root_path = self.library_root()
        else:
            raise ValueError("from_root must be 'project' or 'library'")

        for p in root_path.rglob("*"):
            # Пропуск директорий, которые находятся в исключённых путях
            if any(ex in p.parts for ex in exclude_dirs):
                continue
            if p.name == name and ((include_dirs and p.is_dir()) or (not include_dirs and p.is_file())):
                return p

        return str(root_path / name)


pathfinder = PathFinder()


def init_dirs():
    os.makedirs(f'{b_logs_path()}', exist_ok=True)
    os.makedirs(f'{attachments_path()}', exist_ok=True)
    os.makedirs(f'{static_path()}', exist_ok=True)

    os.makedirs(f'{b_logs_tmp_path()}', exist_ok=True)
    os.makedirs(f'{b_logs_tmp_reports_path()}', exist_ok=True)
    os.makedirs(f'{b_logs_tmp_steps_path()}', exist_ok=True)

    clear_b_logs()
    clear_b_logs_tmp()

    for filename in ("scripts.js", "styles.css", "icon.png"):
        src = Path(pathfinder.library_root()) / f'b_logger/templates/{filename}'
        dst = Path(static_path()) / filename

        if src.exists():
            shutil.copyfile(src, dst)
        else:
            print(f'[BLogger][WARN] static file not found: {src}')


def clear_directory(directory: str, rmdir=False):
    dir_path = Path(directory)
    if not dir_path.exists():
        return

    for entry in dir_path.iterdir():
        if entry.is_file() or entry.is_symlink():
            entry.unlink()
        elif entry.is_dir():
            clear_directory(str(entry))

        if rmdir:
            entry.rmdir()

    if rmdir:
        dir_path.rmdir()


@lru_cache(maxsize=1)
def b_logs_path():
    return pathfinder.find('b_logs')


@lru_cache(maxsize=1)
def attachments_path():
    return pathfinder.find('b_logs/attachments')


@lru_cache(maxsize=1)
def static_path():
    return pathfinder.find('b_logs/static')


@lru_cache(maxsize=1)
def b_logs_tmp_path():
    return pathfinder.find('b_logs_tmp')


@lru_cache(maxsize=1)
def b_logs_tmp_reports_path():
    return pathfinder.find('b_logs_tmp/reports')


@lru_cache(maxsize=1)
def b_logs_tmp_steps_path():
    return pathfinder.find('b_logs_tmp/steps')


def clear_b_logs():
    clear_directory(f'{b_logs_path()}')


def clear_attachments():
    clear_directory(f'{attachments_path()}')


def clear_b_logs_tmp(rmdir=False):
    clear_directory(f'{b_logs_tmp_path()}', rmdir)



from b_logger.config import blog_config
from b_logger.generators.html_gen import HTMLGenerator
from b_logger.generators.report_gen import ReportGenerator
from b_logger.utils.py_addons import BlogPyAddons
from b_logger.utils.paths import *
from b_logger.runtime import RunTime


try:
    import pytest
except ImportError:
    raise ImportError('[BLogger][ERROR] To use b_logger you need pytest to be installed')


try:
    from xdist import is_xdist_controller, get_xdist_worker_id
except ImportError:
    def is_xdist_controller(*args, **kwargs):
        return False

    def get_xdist_worker_id(*args, **kwargs):
        return 'master'


runtime = RunTime()

debug = False


def pytest_addoption(parser):
    group = parser.getgroup('blog')

    BlogPyAddons.add_blog_options(parser, group)


def pytest_configure(config):
    BlogPyAddons.add_blog_markers(config)

    blog_config.apply_cli_options(config)

    blog_config.rootpath = str(config.rootpath)

    if not runtime.run_report.base_url:
        runtime.set_base_url(config.option.blog_base_url)

    if not runtime.run_report.env:
        runtime.set_env(config.option.blog_env)


# def pytest_unconfigure(config):
#     try:
#         report_generator: ReportGenerator = ReportGenerator()
#         html_generator: HTMLGenerator = HTMLGenerator()
#
#         report_generator.generate_combined_report()
#         html_generator.generate_html()
#
#         if not debug:
#             clear_b_logs_tmp(rmdir=True)
#
#     except Exception as e:
#         print(f'[BLogger][ERROR] Unable to generate blog_report: {e}')


@pytest.hookimpl(tryfirst=True)
def pytest_sessionstart(session):
    if _is_main_worker(session):
        init_dirs()

    worker = get_xdist_worker_id(session)
    runtime.run_report.set_worker(worker)


@pytest.hookimpl(trylast=True)
def pytest_sessionfinish(session):
    if not is_xdist_controller(session):
        runtime.run_report.set_end_time()
        runtime.run_report.count_duration()
        runtime.run_report.save_json()

    if _is_main_worker(session):
        try:
            report_generator: ReportGenerator = ReportGenerator()
            html_generator: HTMLGenerator = HTMLGenerator()

            report_generator.generate_combined_report()
            html_generator.generate_html()

            if not debug:
                clear_b_logs_tmp(rmdir=True)

        except Exception as e:
            print(f'[BLogger][ERROR] Unable to generate blog_report: {e}')


def _is_main_worker(session):
    return is_xdist_controller(session) or get_xdist_worker_id(session) == 'master'


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_runtest_protocol(item, nextitem):
    runtime.start_test(item)
    yield
    runtime.finish_test()
    runtime.run_report.save_json()


@pytest.hookimpl(tryfirst=True)
def pytest_runtest_setup(item):
    runtime.step_container.current_stage = 'setup'

    _apply_py_params(item)

    _apply_fixtures(item)

    _apply_markers(item)


@pytest.hookimpl(tryfirst=True)
def pytest_runtest_call(item):
    runtime.step_container.current_stage = 'call'

    _apply_browser(item)


@pytest.hookimpl(tryfirst=True)
def pytest_runtest_teardown(item):
    runtime.step_container.current_stage = 'teardown'


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_runtest_makereport(call, item):

    report = (yield).get_result()

    if call.when not in ["setup", "call"]:
        return

    runtime.process_test_result(report, call, item)

    if report.when == 'setup' and report.outcome == 'passed':
        return

    _apply_py_output(report)

    runtime.process_test_status(report, call, item)


def _apply_py_params(item):
    if hasattr(item, "callspec"):
        params = item.callspec.params
        for param_name, param_value in params.items():
            runtime.apply_info(parameters={param_name: param_value})


def _apply_py_output(report):
    captured_output = {
        "stdout": getattr(report, "capstdout", None),
        "stderr": getattr(report, "capstderr", None),
        "log": getattr(report, "caplog", None),
    }

    for k, v in captured_output.items():
        if v:
            runtime.attach(source=v, name=k)


def _apply_fixtures(item):
    fixtures = item.fixturenames
    runtime.apply_info(fixtures=', '.join(fixtures))


_possible_browser_names = ['driver', 'page', 'selenium_driver', 'driver_init', 'playwright_page']


def _apply_browser(item):
    if not runtime.browser:
        for browser_name in _possible_browser_names:
            if browser_name in item.fixturenames:
                try:
                    browser = item.funcargs.get(browser_name, None)
                    runtime.set_browser(browser)
                except Exception as e:
                    print(f'[BLogger][WARN] Error setting up browser automatically: {e}')


def _apply_markers(item):
    __apply_description_mark(item)
    __apply_info_mark(item)
    __apply_link_mark(item)
    __apply_known_bug_mark(item)


def __apply_description_mark(item):
    try:
        desc = item.get_closest_marker('blog_description').kwargs.get('description')
        runtime.apply_description(desc)

    except AttributeError as e:
        pass


def __apply_info_mark(item):
    try:
        for info in reversed(list(item.iter_markers(name='blog_info'))):
            kwargs = info.kwargs.get('kwargs', {})

            runtime.apply_info(**kwargs)

    except AttributeError as e:
        pass


def __apply_link_mark(item):
    try:
        for link in reversed(list(item.iter_markers(name='blog_link'))):
            kwargs = link.kwargs.get('kwargs', {})

            runtime.apply_link(**kwargs)

    except AttributeError as e:
        pass


def __apply_known_bug_mark(item):
    try:
        for bug in reversed(list(item.iter_markers(name='blog_known_bug'))):
            description = bug.kwargs.get('description')
            url = bug.kwargs.get('url') or None

            if description:
                runtime.apply_known_bug(description, url)
            else:
                print(f'[BLogger][WARN] blog.known_bug usage is incorrect: {bug}')

    except AttributeError as e:
        pass


"""
Just a pytest hooks order hint

pytest_cmdline_main(config)
pytest_collection(session)
pytest_collection_modifyitems(session, config, items)
pytest_collectstart(collector)
pytest_collectreport(report)
pytest_make_collect_report(collector)
pytest_ignore_collect(path, config)
pytest_collect_file(path, parent)

pytest_sessionstart(session)
pytest_collection_finish(session)
pytest_deselected(items)
pytest_itemcollected(item)

pytest_runtestloop(session)

pytest_runtest_protocol(item, nextitem)
├── pytest_runtest_logstart(nodeid, location)
├── pytest_runtest_setup(item)
├── pytest_runtest_makereport(item, call)    # for setup
├── pytest_runtest_logreport(report)    # for setup
├── pytest_runtest_call(item)
├── pytest_runtest_makereport(item, call)    # for call
├── pytest_runtest_logreport(report)    # for call
├── pytest_runtest_teardown(item, nextitem)
├── pytest_runtest_makereport(item, call)    # for teardown
├── pytest_runtest_logreport(report)    # for teardown
└── pytest_runtest_logfinish(nodeid, location)

pytest_exception_interact(node, call, report)
pytest_internalerror(excrepr, excinfo)
pytest_keyboard_interrupt(excinfo)

pytest_terminal_summary(terminalreporter, exitstatus, config)

pytest_sessionfinish(session, exitstatus)
pytest_unconfigure(config)
"""



import uuid
from enum import Enum

from b_logger.utils.basedatamodel import BaseDataModel


# class PrintStatus(str, Enum):
#     PASSED = 'passed'
#     FAILED = 'failed'
#     WARNING = 'warning'
#     NONE = 'none'


class Print(BaseDataModel):
    def __init__(self, data):
        self.id = f'print_{uuid.uuid4()}'
        self.title = data
        self.parent_id = None

    def set_parent_id(self, parent_id):
        self.parent_id = parent_id



import uuid
from typing import Any

from b_logger.entities.attachments import Attachment
from b_logger.entities.steps import StepContainer
from b_logger.entities.statuses import TestStatus
from b_logger.utils.basedatamodel import BaseDataModel
from b_logger.utils.paths import b_logs_tmp_steps_path


# class TestError(BaseDataModel):
#     def __init__(self, exc, stacktrace):
#         self.exception = exc
#         self.stacktrace = stacktrace


# class TestInfo(BaseDataModel):
#     def __init__(self):
#         self.description = None
#         self.parameters = {}
#         self.known_bugs = []
#         self.other = []
#
#     def add_parameter(self, key, value):
#         self.parameters[key]= value


class TestReport(BaseDataModel):

    def __init__(self, module: str = None, name: str = None, originalname: str = None):
        self.module: str = module
        self.name: str = name
        self.originalname: str = originalname
        self.status: TestStatus = TestStatus.NONE
        self.duration: float | None = None
        self.description: str | None = None
        self.info = {}
        self.attachments = []
        self.known_bugs = []
        self.steps = None
        self.error = None
        self.stacktrace = None

    def set_status(self, status: TestStatus):
        self.status = status

    def set_duration(self, duration: float):
        self.duration = duration

    def set_steps(self, steps_id):
        self.steps = steps_id

    def set_error(self, error: str):
        self.error = error

    def set_stacktrace(self, stacktrace: str):
        self.stacktrace = stacktrace

    def set_description(self, description: str):
        self.description = description

    def modify_description(self, description: str):
        self.description += f'\n\n{description}'

    def add_known_bug(self, bug):
        self.known_bugs.append(bug)

    def add_info(self, info: dict):
        for key, value in info.items():
            if key in self.info:
                existing = self.info[key]
                if not isinstance(existing, list):
                    self.info[key] = [existing]
                self.info[key].append(value)
            else:
                self.info[key] = value

    def add_attachment(self, attachment: Attachment):
        self.attachments.append(attachment)

    # def get_steps(self):
    #     steps_path = f'{b_logs_tmp_steps_path()}/{self.steps}.json'
    #     try:
    #         steps = StepContainer.from_json(steps_path)
    #         return steps
    #     except Exception as e:
    #         print(f'[BLogger][WARN] Unable to get steps for {self.name}: {e}')




# class DotDict(dict):
#     """
#     Gives access to dict through dots: obj.key instead of obj['key']
#     """
#
#     def __init__(self, data):
#         super().__init__()
#         for key, value in data.items():
#             self[key] = self._wrap(value)
#
#     def _wrap(self, value):
#         if isinstance(value, dict):
#             return DotDict(value)
#         elif isinstance(value, list):
#             return [self._wrap(item) for item in value]
#         return value
#
#     def __getattr__(self, key):
#         try:
#             return self[key]
#         except KeyError:
#             print(f"No such attribute: {key}")
#             return None
#
#     def __setattr__(self, key, value):
#         self[key] = value




class BlogPyAddons:

    @staticmethod
    def add_blog_options(parser, group):
        group.addoption('--blog_base_url', default=None, action='store', help='')
        group.addoption('--blog_env', default=None, action='store', help='')

    @staticmethod
    def add_blog_markers(config):
        config.addinivalue_line('markers', 'blog_description')
        config.addinivalue_line('markers', 'blog_info')
        config.addinivalue_line('markers', 'blog_link')
        config.addinivalue_line('markers', 'blog_known_bug')
